package java.io;

import sun.misc.SharedSecrets;
import sun.misc.JavaIOFileDescriptorAccess;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;

public final class FileDescriptor
{
    private int fd;
    private long handle;
    private Closeable parent;
    private List<Closeable> otherParents;
    private boolean closed;
    public static final FileDescriptor in;
    public static final FileDescriptor out;
    public static final FileDescriptor err;
    
    public FileDescriptor() {
        this.fd = -1;
        this.handle = -1L;
    }
    
    public boolean valid() {
        return this.handle != -1L || this.fd != -1;
    }
    
    public native void sync() throws SyncFailedException;
    
    private static native void initIDs();
    
    private static native long set(int p0);
    
    private static FileDescriptor standardStream(int n) {
        FileDescriptor fileDescriptor = new FileDescriptor();
        fileDescriptor.handle = set(n);
        return fileDescriptor;
    }
    
    synchronized void attach(Closeable parent) {
        if (this.parent == null) {
            this.parent = parent;
        }
        else if (this.otherParents == null) {
            (this.otherParents = new ArrayList()).add(this.parent);
            this.otherParents.add(parent);
        }
        else {
            this.otherParents.add(parent);
        }
    }
    
    synchronized void closeAll(Closeable closeable) throws IOException {
        if (!this.closed) {
            this.closed = true;
            IOException exception = null;
            try {
                Throwable t = null;
                try {
                    if (this.otherParents != null) {
                        for (Closeable closeable2 : this.otherParents) {
                            try {
                                closeable2.close();
                            }
                            catch (IOException exception2) {
                                if (exception == null) {
                                    exception = exception2;
                                }
                                else {
                                    exception.addSuppressed(exception2);
                                }
                            }
                        }
                    }
                    if (closeable != null) {
                        if (t != null) {
                            try {
                                closeable.close();
                            }
                            catch (Throwable exception3) {
                                t.addSuppressed(exception3);
                            }
                        }
                        else {
                            closeable.close();
                        }
                    }
                }
                catch (Throwable t2) {
                    t = t2;
                    throw t2;
                }
                finally {
                    if (closeable != null) {
                        if (t != null) {
                            try {
                                closeable.close();
                            }
                            catch (Throwable exception4) {
                                t.addSuppressed(exception4);
                            }
                        }
                        else {
                            closeable.close();
                        }
                    }
                }
            }
            catch (IOException ex) {
                if (exception != null) {
                    ex.addSuppressed(exception);
                }
                exception = ex;
            }
            finally {
                if (exception != null) {
                    throw exception;
                }
            }
        }
    }
    
    static {
        initIDs();
        SharedSecrets.setJavaIOFileDescriptorAccess(new JavaIOFileDescriptorAccess() {
            public void set(FileDescriptor fileDescriptor, int n) {
                fileDescriptor.fd = n;
            }
            
            public int get(FileDescriptor fileDescriptor) {
                return fileDescriptor.fd;
            }
            
            public void setHandle(FileDescriptor fileDescriptor, long n) {
                fileDescriptor.handle = n;
            }
            
            public long getHandle(FileDescriptor fileDescriptor) {
                return fileDescriptor.handle;
            }
        });
        in = standardStream(0);
        out = standardStream(1);
        err = standardStream(2);
    }
}
